@@ -391,6 +391,9 @@ on_screen_changed (CcDisplayPanel *panel)
   GtkSizeGroup *sizegroup;
   GList *sorted_outputs = NULL, *l;
 
+  if (priv->dialog)
+    gtk_dialog_response (GTK_DIALOG (priv->dialog), GTK_RESPONSE_NONE);
+
   gnome_rr_screen_refresh (priv->screen, NULL);
 
   current = gnome_rr_config_new_current (priv->screen, NULL);
@@ -1569,12 +1572,7 @@ show_arrange_displays_dialog (GtkButton      *button,
   response = gtk_dialog_run (GTK_DIALOG (priv->dialog));
   if (response == GTK_RESPONSE_ACCEPT)
     apply_current_configuration (panel);
-  else if (response == GTK_RESPONSE_NONE)
-    {
-      /* panel is being destroyed */
-      return;
-    }
-  else
+  else if (response != GTK_RESPONSE_NONE)
     {
       /* re-read the previous configuration */
       on_screen_changed (panel);
@@ -2113,9 +2111,10 @@ show_setup_dialog (CcDisplayPanel *panel)
   gtk_widget_set_halign (label, GTK_ALIGN_START);
 
   /* resolution combo box */
-  res_model = gtk_list_store_new (2, G_TYPE_STRING, GNOME_TYPE_RR_MODE);
-  renderer = gtk_cell_renderer_text_new ();
+  res_model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
   priv->res_combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (res_model));
+  g_object_unref (res_model);
+  renderer = gtk_cell_renderer_text_new ();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->res_combo), renderer, TRUE);
   gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (priv->res_combo), renderer, "text", 0);
   g_signal_connect (priv->res_combo, "changed", G_CALLBACK (res_combo_changed),
@@ -2298,12 +2297,7 @@ show_setup_dialog (CcDisplayPanel *panel)
 
       apply_current_configuration (panel);
     }
-  else if (response == GTK_RESPONSE_NONE)
-    {
-      /* panel is being destroyed */
-      return;
-    }
-  else
+  else if (response != GTK_RESPONSE_NONE)
     {
       /* changes cancelled, so re-read the current configuration */
       on_screen_changed (panel);
