@@ -625,6 +625,7 @@ connect_ip4_page (CEPageIP4 *page)
                                            -1);
 
         gtk_combo_box_set_model (page->method, GTK_TREE_MODEL (store));
+        g_object_unref (G_OBJECT (store));
 
         method = IP4_METHOD_AUTO;
         if (g_strcmp0 (str_method, NM_SETTING_IP4_CONFIG_METHOD_LINK_LOCAL) == 0) {
@@ -704,7 +705,11 @@ ui_to_setting (CEPageIP4 *page)
         }
 
         addresses = g_ptr_array_new_with_free_func (free_addr);
-        children = gtk_container_get_children (GTK_CONTAINER (page->address_list));
+        if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
+                children = gtk_container_get_children (GTK_CONTAINER (page->address_list));
+        else
+                children = NULL;
+
         for (l = children; l; l = l->next) {
                 GtkWidget *row = l->data;
                 GtkEntry *entry;
@@ -774,7 +779,12 @@ ui_to_setting (CEPageIP4 *page)
         }
 
         dns_servers = g_array_new (FALSE, FALSE, sizeof (guint));
-        children = gtk_container_get_children (GTK_CONTAINER (page->dns_list));
+        if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO) ||
+            g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
+                children = gtk_container_get_children (GTK_CONTAINER (page->dns_list));
+        else
+                children = NULL;
+
         for (l = children; l; l = l->next) {
                 GtkWidget *row = l->data;
                 GtkEntry *entry;
@@ -802,9 +812,13 @@ ui_to_setting (CEPageIP4 *page)
         }
         g_list_free (children);
 
-
         routes = g_ptr_array_new_with_free_func (free_addr);
-        children = gtk_container_get_children (GTK_CONTAINER (page->routes_list));
+        if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO) ||
+            g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
+                children = gtk_container_get_children (GTK_CONTAINER (page->routes_list));
+        else
+                children = NULL;
+
         for (l = children; l; l = l->next) {
                 GtkWidget *row = l->data;
                 GtkEntry *entry;
